@@ -22,6 +22,7 @@
 
 import os.path
 import gobject
+import time
 import gtk
 import gst
 import gst.interfaces
@@ -63,13 +64,11 @@ class PitiviViewer(gtk.VBox):
         # TODO remove/replace the signal when closing/opening projects
         instance.PiTiVi.current.sources.connect("tmp_is_ready", self._tmpIsReadyCb)
 
-        # Only set the check time timeout in certain cases...
-        self.checktimeoutid = 0
-        self.positionChangedCallbacks = []
         instance.PiTiVi.connect("new-project", self._newProjectCb)
         instance.PiTiVi.playground.connect("current-state", self._currentStateCb)
         instance.PiTiVi.playground.connect("bin-added", self._binAddedCb)
         instance.PiTiVi.playground.connect("bin-removed", self._binRemovedCb)
+        instance.PiTiVi.playground.connect("position", self._playgroundPositionCb)
 
         instance.PiTiVi.current.settings.connect("settings-changed",
                                                  self._settingsChangedCb)
@@ -195,9 +194,11 @@ class PitiviViewer(gtk.VBox):
         vsinkthread = gst.Bin('vsinkthread')
         vqueue = gst.element_factory_make('queue')
         cspace = gst.element_factory_make('ffmpegcolorspace')
-        vsinkthread.add(self.videosink, vqueue, cspace)
+        vscale = gst.element_factory_make('videoscale')
+        vsinkthread.add(self.videosink, vqueue, vscale, cspace)
         vqueue.link(self.videosink)
-        cspace.link(vqueue)
+        cspace.link(vscale)
+        vscale.link(vqueue)
         vsinkthread.videosink = self.videosink
         vsinkthread.add_pad(gst.GhostPad("sink", cspace.get_pad('sink')))
 
@@ -239,11 +240,8 @@ class PitiviViewer(gtk.VBox):
     def _sliderButtonPressCb(self, slider, unused_event):
         gst.info("button pressed")
         self.moving_slider = True
-        if self.checktimeoutid:
-            gobject.source_remove(self.checktimeoutid)
-            self.checktimeoutid = 0
         self.valuechangedid = slider.connect("value-changed", self._sliderValueChangedCb)
-        instance.PiTiVi.playground.current.set_state(gst.STATE_PAUSED)
+        instance.PiTiVi.playground.pause()
         return False
 
     def _sliderButtonReleaseCb(self, slider, unused_event):
@@ -252,11 +250,11 @@ class PitiviViewer(gtk.VBox):
         if self.valuechangedid:
             slider.disconnect(self.valuechangedid)
             self.valuechangedid = 0
-        if not self.checktimeoutid:
-            gst.debug("adding checktime again")
-            self.checktimeoutid = gobject.timeout_add(300, self._checkTimeCb)
         # revert to previous state
-        instance.PiTiVi.playground.current.set_state(self.currentState)
+        if self.currentState == gst.STATE_PAUSED:
+            instance.PiTiVi.playground.pause()
+        else:
+            instance.PiTiVi.playground.play()
         return False
 
     def _sliderValueChangedCb(self, slider):
@@ -264,7 +262,6 @@ class PitiviViewer(gtk.VBox):
         value = long(slider.get_value())
         gst.info(time_to_string(value))
         self._doSeek(value)
-        #instance.PiTiVi.playground.seekInCurrent(value)
 
     def _sliderScrollCb(self, unused_slider, event):
         # calculate new seek position
@@ -275,7 +272,6 @@ class PitiviViewer(gtk.VBox):
             else:
                 seekvalue = min(self.current_time + gst.SECOND / 2, instance.PiTiVi.playground.current.length)
             self._doSeek(seekvalue)
-            #instance.PiTiVi.playground.seekInCurrent(seekvalue)
         else:
             # frame scrolling, frame by frame
             gst.info("scroll direction:%s" % event.direction)
@@ -286,7 +282,6 @@ class PitiviViewer(gtk.VBox):
                 gst.info("scrolling forward")
                 seekvalue = min(self.current_frame + 1, instance.PiTiVi.playground.current.length)
             self._doSeek(seekvalue, gst.FORMAT_DEFAULT)
-            #instance.PiTiVi.playground.seekInCurrent(seekvalue, gst.FORMAT_DEFAULT)
 
     def _seekTimeoutCb(self):
         self.currentlySeeking = False
@@ -302,33 +297,6 @@ class PitiviViewer(gtk.VBox):
         elif format == gst.FORMAT_TIME:
             self.requested_time = value
 
-    ## timeout functions for checking current time
-
-    ## TODO : check_time timeout should in fact be in the playground to be more generic
-    def _checkTimeCb(self):
-        # check time callback
-        gst.log("checking time")
-        if instance.PiTiVi.playground.current == instance.PiTiVi.playground.default:
-            return True
-        # don't check time if the timeline is not playing
-        cur = self.current_time
-        currentframe = self.current_frame
-        if True: #not isinstance(instance.PiTiVi.playground.current, SmartTimelineBin):
-            pending, state, result = instance.PiTiVi.playground.current.get_state(10)
-            if not state in [gst.STATE_PAUSED, gst.STATE_PLAYING]:
-                return
-            try:
-                cur, format = instance.PiTiVi.playground.current.query_position(gst.FORMAT_TIME)
-            except:
-                instance.PiTiVi.playground.current.warning("couldn't get position")
-                cur = 0
-
-        # if the current_time or the length has changed, update time
-        if not float(instance.PiTiVi.playground.current.length) == self.posadjust.upper or not cur == self.current_time or not currentframe == self.current_frame:
-            self.posadjust.upper = float(instance.PiTiVi.playground.current.length)
-            self._newTime(cur, currentframe)
-        return True
-
     def _newTime(self, value, frame=-1):
         gst.info("value:%s, frame:%d" % (gst.TIME_ARGS(value), frame))
         self.current_time = value
@@ -336,19 +304,8 @@ class PitiviViewer(gtk.VBox):
         self.timelabel.set_markup("<tt>%s / %s</tt>" % (time_to_string(value), time_to_string(instance.PiTiVi.playground.current.length)))
         if not self.moving_slider:
             self.posadjust.set_value(float(value))
-        if isinstance(instance.PiTiVi.playground.current, SmartTimelineBin):
-            self._triggerTimelinePositionCallbacks(value, frame)
         return False
 
-    def _triggerTimelinePositionCallbacks(self, value, frame = -1):
-        for callback in self.positionChangedCallbacks:
-            callback(value, frame)
-
-    def addTimelinePositionCallback(self, function):
-        """ Set the function that will be called whenever the timeline position has changed """
-        self.positionChangedCallbacks.append(function)
-
-
     ## gtk.ComboBox callbacks for sources
 
     def _comboboxChangedCb(self, cbox):
@@ -409,9 +366,14 @@ class PitiviViewer(gtk.VBox):
 
 
     ## Control gtk.Button callbacks
+
+    def _encodingDialogDestroyCb(self, unused_dialog):
+        instance.PiTiVi.gui.set_sensitive(True)
         
     def _recordCb(self, unused_button):
         win = EncodingDialog(instance.PiTiVi.current)
+        win.window.connect("destroy", self._encodingDialogDestroyCb)
+        instance.PiTiVi.gui.set_sensitive(False)
         win.show()
 
     def _rewindCb(self, unused_button):
@@ -437,6 +399,9 @@ class PitiviViewer(gtk.VBox):
 
     ## Playground callbacks
     
+    def _playgroundPositionCb(self, playground, smartbin, pos):
+        self._newTime(pos)
+
     def _currentPlaygroundChangedCb(self, playground, smartbin):
         if not smartbin == playground.default:
             if isinstance(smartbin, SmartTimelineBin):
@@ -485,14 +450,8 @@ class PitiviViewer(gtk.VBox):
     def _currentStateCb(self, playground, state):
         gst.info("current state changed : %s" % state)
         if state == int(gst.STATE_PLAYING):
-            if not isinstance(playground.current, SmartDefaultBin) and not self.checktimeoutid:
-                gst.info("adding checktime")
-                self.checktimeoutid = gobject.timeout_add(300, self._checkTimeCb)
             self.playpause_button.setPause()
         elif state == int(gst.STATE_PAUSED):
-            if not isinstance(playground.current, SmartDefaultBin) and not self.checktimeoutid:
-                gst.info("adding checktime")
-                self.checktimeoutid = gobject.timeout_add(300, self._checkTimeCb)
             self.playpause_button.setPlay()
 
 
@@ -569,9 +528,11 @@ class EncodingDialog(GladeWindow):
         self.eosid = self.bus.connect("message::eos", self._eosCb)
         self.outfile = None
         self.progressbar = self.widgets["progressbar"]
-        self.timeoutid = None
+        self.cancelbutton = self.widgets["cancelbutton"]
+        self.positionhandler = 0
         self.rendering = False
         self.settings = project.settings
+        self.timestarted = 0
 
     def _fileButtonClickedCb(self, button):
         
@@ -589,11 +550,23 @@ class EncodingDialog(GladeWindow):
             button.set_label(os.path.basename(self.outfile))
         dialog.destroy()
 
+    def _positionCb(self, playground, smartbin, position):
+        timediff = time.time() - self.timestarted
+        self.progressbar.set_fraction(float(position) / float(self.bin.length))
+        if timediff > 5.0:
+            # only display ETA after 5s in order to have enough averaging
+            totaltime = (timediff * float(self.bin.length) / float(position)) - timediff
+            self.progressbar.set_text("Finished in %dm%ds" % (int(totaltime) / 60,
+                                                              int(totaltime) % 60))
+
     def _recordButtonClickedCb(self, unused_button):
         if self.outfile and not self.rendering:
             if self.bin.record(self.outfile, self.settings):
-                self.timeoutid = gobject.timeout_add(400, self._timeoutCb)
+                self.timestarted = time.time()
+                self.positionhandler = instance.PiTiVi.playground.connect('position', self._positionCb)
                 self.rendering = True
+                self.cancelbutton.set_label("gtk-cancel")
+                self.progressbar.set_text("Rendering")
             else:
                 self.progressbar.set_text("Couldn't start rendering")
 
@@ -605,20 +578,6 @@ class EncodingDialog(GladeWindow):
             self.settings = dialog.getSettings()
         dialog.destroy()
 
-    def _timeoutCb(self):
-        result, state, pending = self.bin.get_state(0)
-        if state == gst.STATE_PLAYING and self.rendering:
-            # check time
-            value, format = self.bin.query_position(gst.FORMAT_TIME)
-            # set progresbar to percentage
-            self.progressbar.set_fraction(float(value) / float(self.bin.length))
-            
-            # display current time in progressbar
-            self.progressbar.set_text(time_to_string(value))
-            return True
-        self.timeoutid = False
-        return False
-
     def do_destroy(self):
         gst.debug("cleaning up...")
         self.bus.remove_signal_watch()
@@ -626,14 +585,18 @@ class EncodingDialog(GladeWindow):
 
     def _eosCb(self, unused_bus, unused_message):
         self.rendering = False
-        self.progressbar.set_text("Rendering Finished")
+        if self.positionhandler:
+            instance.PiTiVi.playground.disconnect(self.positionhandler)
+            self.positionhandler = 0
+        self.progressbar.set_text("Rendering Complete")
         self.progressbar.set_fraction(1.0)
-        gobject.source_remove(self.timeoutid)
+        self.cancelbutton.set_label("gtk-close")
         
     def _cancelButtonClickedCb(self, unused_button):
         self.bin.stopRecording()
-        if self.timeoutid:
-            gobject.source_remove(self.timeoutid)
-            self.timeoutid = None
+        if self.positionhandler:
+            instance.PiTiVi.playground.disconnect(self.positionhandler)
+            self.positionhandler = 0
+        instance.PiTiVi.playground.pause()
         self.destroy()
 
